/*
 Simpel en recht toe recht aan programma om de Si5351A en ADF435x in te stellen op de gewenste frequenties.
 Si5351A Clk0 = 25 MHZ voor een LNB (Oudere octagon meestal 27 MHz)
 Si5351A Clk1 = 10 MHz voor de upconverter
 Si5351A Clk2 = 10 MHz als referentie voor het extra ADF435x bordje

 Een en ander moet nog wel netter gemaakt worden en kiesbaar via de optie jumpers maar voor nu werkt het.
 Andere frequenties of drive levels kunnen in de vershillend regels aangepast worden.
 De referentie voor de ADF435x op 10 MHz laten want de formules verwachten deze waarde.

 Veel dank gaat uit naar de makers van de 'Etherkit Si5351' library en oe6ocg voor zijn voorbeeld pragramma voor de ADF435x.
 Omdat dat beide stukjes GPL zijn is dit stukje broddelwerk dat ook.

 2020 - Ad Sekeris PE1GIJ

 Versie: 0.0.2 - 18-06-2020
*/
 
#include "si5351.h"   // Bibliotheek 'Etherkit Si5351'
#include "Wire.h"     // Bibliotheek voor I2C
#include "SPI.h"      // Bibliotheek voor SPI

unsigned long long AdfFreq = 307500000;
unsigned long long Clk0Freq = 2500000000ULL;
unsigned long long Clk1Freq = 1000000000ULL;
unsigned long long Clk2Freq = 1000000000ULL;

byte AdfRfOut = 1;
byte Clk0RfOut = 1;
byte Clk1RfOut = 1;
byte Clk2RfOut = 1;

byte AdfLevel = 0;   // Power level: 0 = -4dBm / 1 = -1dBm / 2 = +2dBm / 3 = +5dBm
  // Uitgangs vermogen per kanaal zonder filtering
  // 0 = SI5351_DRIVE_2MA = 2mA in 50 Ohm ong. -1dBm
  // 1 = SI5351_DRIVE_4MA = 4mA in 50 Ohm ong. 5dBm
  // 2 = SI5351_DRIVE_6MA = 6mA in 50 Ohm ong. 7dBm
  // 3 = SI5351_DRIVE_8MA = 8mA in 50 Ohm ong. 8dBm
byte Clk0Level = 0;
byte Clk1Level = 0;
byte Clk2Level = 0;

// Referentie voor de Si5351 chip
#define si_xtal  1000000000ULL  
// Referentie voor de ADF4351 chip
//#define xtal  25000000    // Adf4351 Refin aangesloten op een 25 MHz uitgang van de Si5351 chip
#define xtal  10000000  // ADF43512 Refin parallel aan de Refin van de GPSDO1

//Pro micro
//const int slaveSelectPin = 10;  //SPI-SS om de ADF435x te selecteren
//Mega2560 GPSDO
//const int slaveSelectPin = 47;  //SPI-SS om de ADF435x te selecteren
// Load the functions for the ADF4351
#define SCLK 15  // SPI bus connections
#define MISO 14
#define MOSI 16
#define SS 10  //SPI-SS om de ADF435x te selecteren
unsigned long AdfStep = 1000;
#include "ADF4351_func.h" // synth library
ADF4351 dds;  // instantiate "dds" object
Si5351 si5351;  // Init Etherkit Si5351


void setup()
{
  // Geef de 3 optie ingangen een pull-up weerstand
  pinMode(7, INPUT_PULLUP);  //JP7
  pinMode(8, INPUT_PULLUP);  //JP6
  pinMode(9, INPUT_PULLUP);  //JP5
  // Start de Si5351
  // Si5351A met extern clock signaal van 10 MHz, geen correctie
  si5351.init(SI5351_CRYSTAL_LOAD_0PF, 10000000, 0);
  si5351.set_clock_disable(SI5351_CLK0, SI5351_CLK_DISABLE_HI_Z);
  si5351.set_clock_disable(SI5351_CLK1, SI5351_CLK_DISABLE_HI_Z);
  si5351.set_clock_disable(SI5351_CLK2, SI5351_CLK_DISABLE_HI_Z);
  si5351.set_int(SI5351_CLK0, 1);
  si5351.set_int(SI5351_CLK1, 1);
  si5351.set_int(SI5351_CLK2, 1);

  // Uitgangs vermogen per kanaal zonder filtering
  // SI5351_DRIVE_2MA = 2mA in 50 Ohm ong. -1dBm
  // SI5351_DRIVE_4MA = 4mA in 50 Ohm ong. 5dBm
  // SI5351_DRIVE_6MA = 6mA in 50 Ohm ong. 7dBm
  // SI5351_DRIVE_8MA = 8mA in 50 Ohm ong. 8dBm

  if (Clk0RfOut == 1) { 
    si5351.output_enable(SI5351_CLK0, 1);
    si5351.drive_strength(SI5351_CLK0, Clk0Level);
  }
  else {
    si5351.output_enable(SI5351_CLK0, 0);
    si5351.set_clock_pwr(SI5351_CLK0, 0);
  }
  if (Clk1RfOut == 1) { 
    si5351.output_enable(SI5351_CLK1, 1);
    si5351.drive_strength(SI5351_CLK1, Clk1Level);
  }
  else {
    si5351.output_enable(SI5351_CLK1, 0);
    si5351.set_clock_pwr(SI5351_CLK1, 0);
  }
  if (Clk2RfOut == 1) { 
    si5351.output_enable(SI5351_CLK2, 1);
    si5351.drive_strength(SI5351_CLK2, Clk2Level);
  }
  else {
    si5351.output_enable(SI5351_CLK2, 0);
    si5351.set_clock_pwr(SI5351_CLK2, 0);
  }

  if ((Clk0RfOut == 1)&&(Clk0Freq != si_xtal)) {
    si5351.set_freq(Clk0Freq, SI5351_CLK0);
  }
  else if ((Clk0RfOut == 1)&&(Clk0Freq == si_xtal)){
    si5351.set_clock_fanout(SI5351_FANOUT_XO, 1);
    si5351.set_clock_source(SI5351_CLK0, SI5351_CLK_SRC_XTAL);
  }
  if ((Clk1RfOut == 1)&&(Clk1Freq != si_xtal)) {
    si5351.set_freq(Clk1Freq, SI5351_CLK1);
  }
  else if ((Clk1RfOut == 1)&&(Clk1Freq == si_xtal)){
    si5351.set_clock_fanout(SI5351_FANOUT_XO, 1);
    si5351.set_clock_source(SI5351_CLK1, SI5351_CLK_SRC_XTAL);
  }
  if ((Clk2RfOut == 1)&&(Clk2Freq != si_xtal)) {
    si5351.set_freq(Clk2Freq, SI5351_CLK2);
  }
  else if ((Clk2RfOut == 1)&&(Clk2Freq == si_xtal)){
    si5351.set_clock_fanout(SI5351_FANOUT_XO, 1);
    si5351.set_clock_source(SI5351_CLK2, SI5351_CLK_SRC_XTAL);
  }

  //si5351.output_enable(SI5351_CLK0, Clk0RfOut); 
  //si5351.output_enable(SI5351_CLK1, Clk1RfOut); 
  //si5351.output_enable(SI5351_CLK2, Clk2RfOut);     

  si5351.update_status();

  dds.init();                     // init dds
  dds.setFreq(AdfFreq, AdfStep);  // Set frequency (35MHz - 4.4GHz) and channel spacing (1kHz)
  delay(100);                     // Settling time
  dds.update();                   // Make new settings active
  dds.setRFPower(AdfLevel);       // Power level: 0 = -4dBm / 1 = -1dBm / 2 = +2dBm / 3 = +5dBm
  dds.setRFOut(AdfRfOut);         // Output on
  dds.update();                   // Make new settings active

  delay(500);
}

void loop()
{
  // Na het instellen van de chips staat de arduino niets te doen
}
